@@ -37,7 +37,7 @@ def init_model(config, checkpoint=None, device='cuda:0', cfg_options=None):
     model.init_weights()
     if checkpoint is not None:
         checkpoint = load_checkpoint(model, checkpoint, map_location='cpu')
-        if 'CLASSES' in checkpoint['meta']:
+        if 'meta' in checkpoint and 'CLASSES' in checkpoint['meta']:
             model.CLASSES = checkpoint['meta']['CLASSES']
     if not hasattr(model, 'CLASSES'):
         if hasattr(model, 'detector') and hasattr(model.detector, 'CLASSES'):
