@@ -13,6 +13,11 @@ from mmdet.datasets import DATASETS
 from mmtrack.core import interpolate_tracks, results2outs
 from .coco_video_dataset import CocoVideoDataset
 
+try:
+    import trackeval
+except ImportError:
+    trackeval = None
+
 
 @DATASETS.register_module()
 class MOTChallengeDataset(CocoVideoDataset):
@@ -185,7 +190,7 @@ class MOTChallengeDataset(CocoVideoDataset):
                           self.data_infos[inds[i]:inds[i + 1]],
                           f'{resfiles[metric]}/{names[i]}.txt')
 
-        return resfiles, names, tmp_dir
+        return resfile_path, resfiles, names, tmp_dir
 
     def format_track_results(self, results, infos, resfile):
         """Format tracking results."""
@@ -240,6 +245,81 @@ class MOTChallengeDataset(CocoVideoDataset):
                         f'{(y2-y1):.3f},{conf:.3f}\n')
             f.close()
 
+    def get_benchmark(self):
+        """Get benchmark from upeper/lower-case image prefix."""
+        BENCHMARKS = ['MOT15', 'MOT16', 'MOT17', 'MOT20']
+        for benchamrk in BENCHMARKS:
+            if benchamrk in self.img_prefix.upper():
+                return benchamrk
+
+    def get_dataset_cfg_for_hota(self, tracker_folder, seqmap):
+        """Get default configs for trackeval.datasets.MotChallenge2DBox.
+
+        Args:
+            tracker_folder (str): the name of the tracker folder
+            seqmap (str): the file that contains the sequence of video names
+
+        Returns:
+            Dataset Configs for MotChallenge2DBox.
+        """
+        dataset_config = dict(
+            # Location of GT data
+            GT_FOLDER=self.img_prefix,
+            # Trackers location
+            TRACKERS_FOLDER=tracker_folder,
+            # Where to save eval results
+            # (if None, same as TRACKERS_FOLDER)
+            OUTPUT_FOLDER=None,
+            # Use 'track' as the default tracker
+            TRACKERS_TO_EVAL=['track'],
+            # Option values: ['pedestrian']
+            CLASSES_TO_EVAL=list(self.CLASSES),
+            # Option Values: 'MOT17', 'MOT16', 'MOT20', 'MOT15'
+            BENCHMARK=self.get_benchmark(),
+            # Option Values: 'train', 'test'
+            SPLIT_TO_EVAL='train',
+            # Whether tracker input files are zipped
+            INPUT_AS_ZIP=False,
+            # Whether to print current config
+            PRINT_CONFIG=True,
+            # Whether to perform preprocessing
+            # (never done for MOT15)
+            DO_PREPROC=False if 'MOT15' in self.img_prefix else True,
+            # Tracker files are in
+            # TRACKER_FOLDER/tracker_name/TRACKER_SUB_FOLDER
+            TRACKER_SUB_FOLDER='',
+            # Output files are saved in
+            # OUTPUT_FOLDER/tracker_name/OUTPUT_SUB_FOLDER
+            OUTPUT_SUB_FOLDER='',
+            # Names of trackers to display
+            # (if None: TRACKERS_TO_EVAL)
+            TRACKER_DISPLAY_NAMES=None,
+            # Where seqmaps are found
+            # (if None: GT_FOLDER/seqmaps)
+            SEQMAP_FOLDER=None,
+            # Directly specify seqmap file
+            # (if none use seqmap_folder/benchmark-split_to_eval)
+            SEQMAP_FILE=seqmap,
+            # If not None, specify sequences to eval
+            # and their number of timesteps
+            SEQ_INFO=None,
+            # '{gt_folder}/{seq}/gt/gt.txt'
+            GT_LOC_FORMAT='{gt_folder}/{seq}/gt/gt.txt',
+            # If False, data is in GT_FOLDER/BENCHMARK-SPLIT_TO_EVAL/ and in
+            # TRACKERS_FOLDER/BENCHMARK-SPLIT_TO_EVAL/tracker/
+            # If True, the middle 'benchmark-split' folder is skipped for both.
+            SKIP_SPLIT_FOL=True,
+        )
+
+        if 'half-train' in self.ann_file:
+            dataset_config[
+                'GT_LOC_FORMAT'] = '{gt_folder}/{seq}/gt/gt_half-train.txt'
+        elif 'half-val' in self.ann_file:
+            dataset_config[
+                'GT_LOC_FORMAT'] = '{gt_folder}/{seq}/gt/gt_half-val.txt'
+
+        return dataset_config
+
     def evaluate(self,
                  results,
                  metric='track',
@@ -278,7 +358,7 @@ class MOTChallengeDataset(CocoVideoDataset):
                 raise KeyError(f'metric {metric} is not supported.')
 
         if 'track' in metrics:
-            resfiles, names, tmp_dir = self.format_results(
+            resfile_path, resfiles, names, tmp_dir = self.format_results(
                 results, resfile_path, metrics)
             print_log('Evaluate CLEAR MOT results.', logger=logger)
             distth = 1 - track_iou_thr
@@ -311,17 +391,60 @@ class MOTChallengeDataset(CocoVideoDataset):
                 names=names,
                 metrics=mm.metrics.motchallenge_metrics,
                 generate_overall=True)
-            str_summary = mm.io.render_summary(
-                summary,
-                formatters=mh.formatters,
-                namemap=mm.io.motchallenge_metric_names)
-            print(str_summary)
+
+            if trackeval is None:
+                raise ImportError(
+                    'Please run'
+                    'pip install git+https://github.com/JonathonLuiten/TrackEval.git'  # noqa
+                    'to manually install trackeval')
+
+            seqmap = osp.join(resfile_path, 'videoseq.txt')
+            with open(seqmap, 'w') as f:
+                f.write('name\n')
+                for name in names:
+                    f.write(name + '\n')
+                f.close
+
+            eval_config = trackeval.Evaluator.get_default_eval_config()
+
+            # tracker's name is set to 'track',
+            # so this word needs to be splited out
+            output_folder = resfiles['track'].rsplit(os.sep, 1)[0]
+
+            dataset_config = self.get_dataset_cfg_for_hota(
+                output_folder, seqmap)
+
+            evaluator = trackeval.Evaluator(eval_config)
+            dataset = [trackeval.datasets.MotChallenge2DBox(dataset_config)]
+            hota_metrics = [
+                trackeval.metrics.HOTA(dict(METRICS=['HOTA'], THRESHOLD=0.5))
+            ]
+            output_res, _ = evaluator.evaluate(dataset, hota_metrics)
+
+            # modify HOTA results sequence according to summary list,
+            # indexes of summary are sequence names and 'OVERALL'
+            # while for hota they are sequence names and 'COMBINED_SEQ'
+            seq_list = list(summary.index)
+            seq_list.append('COMBINED_SEQ')
+
+            hota = [
+                np.average(output_res['MotChallenge2DBox']['track'][seq]
+                           ['pedestrian']['HOTA']['HOTA']) for seq in seq_list
+                if 'OVERALL' not in seq
+            ]
 
             eval_results.update({
                 mm.io.motchallenge_metric_names[k]: v['OVERALL']
                 for k, v in summary.to_dict().items()
             })
+            eval_results['HOTA'] = hota[-1]
 
+            summary['HOTA'] = hota
+            str_summary = mm.io.render_summary(
+                summary,
+                formatters=mh.formatters,
+                namemap=mm.io.motchallenge_metric_names)
+            print(str_summary)
             if tmp_dir is not None:
                 tmp_dir.cleanup()
 
