@@ -1,4 +1,5 @@
 # Copyright (c) OpenMMLab. All rights reserved.
 from .siamrpn import SiamRPN
+from .stark import Stark
 
-__all__ = ['SiamRPN']
+__all__ = ['SiamRPN', 'Stark']
