@@ -2,5 +2,9 @@
 from .roi_embed_head import RoIEmbedHead
 from .roi_track_head import RoITrackHead
 from .siamese_rpn_head import CorrelationHead, SiameseRPNHead
+from .stark_head import CornerPredictorHead, StarkHead
 
-__all__ = ['CorrelationHead', 'SiameseRPNHead', 'RoIEmbedHead', 'RoITrackHead']
+__all__ = [
+    'CorrelationHead', 'SiameseRPNHead', 'RoIEmbedHead', 'RoITrackHead',
+    'StarkHead', 'CornerPredictorHead'
+]
